@@ -1000,7 +1000,8 @@ goal_fun_creator_egarch <- function(
 setMethod("fEGarch_fit", "egarch_type_spec",
   function(spec = egarch_spec(), rt, drange = c(0, 1), meanspec = mean_spec(), Drange = c(0, 1),
     n_test = 0, start_pars = NULL, LB = NULL, UB = NULL, control = list(), parallel = TRUE,
-    ncores = max(1, future::availableCores() - 1), trunc = "none", presample = 50, Prange = c(1, 5)) {
+    ncores = max(1, future::availableCores() - 1), trunc = "none", presample = 50, Prange = c(1, 5),
+    skip_vcov = FALSE) {
 
   # Make rt have sample variance 1; more robust for
   # differently scaled data; results like parameters
@@ -1346,7 +1347,8 @@ setMethod("fEGarch_fit", "egarch_type_spec",
     P_sel = P_sel,
     cond_d = cond_d,
     par_names = par_names,
-    model_type = model_type
+    model_type = model_type,
+    skip_vcov = skip_vcov
   )
 
   # Create rescaled final parameters and results
@@ -1593,7 +1595,7 @@ goal_fun_creator_loggarch <- function(
 #
 #
 setMethod("fEGarch_fit", "loggarch_type_spec",
-  function(spec = loggarch_spec(), rt, drange = c(0, 1), meanspec = mean_spec(), Drange = c(0, 1), n_test = 0, start_pars = NULL, LB = NULL, UB = NULL, control = list(), parallel = TRUE, ncores = max(1, future::availableCores() - 1), trunc = "none", presample = 50, Prange = c(1, 5)) {
+  function(spec = loggarch_spec(), rt, drange = c(0, 1), meanspec = mean_spec(), Drange = c(0, 1), n_test = 0, start_pars = NULL, LB = NULL, UB = NULL, control = list(), parallel = TRUE, ncores = max(1, future::availableCores() - 1), trunc = "none", presample = 50, Prange = c(1, 5), skip_vcov = skip_vcov) {
 
   # Make rt have sample variance 1; more robust for
   # differently scaled data; results like parameters
@@ -1914,7 +1916,8 @@ setMethod("fEGarch_fit", "loggarch_type_spec",
     P_sel = P_sel,
     cond_d = cond_d,
     par_names = par_names,
-    model_type = model_type
+    model_type = model_type,
+    skip_vcov = skip_vcov
   )
 
   # Create rescaled final parameters and results
