@@ -204,6 +204,9 @@ goal_fun_creator_fiaparch <- function(
 #'@param fix_delta let the parameter \eqn{\delta} be either free (\code{fix_delta = NA}),
 #'fix it to 1 (\code{fix_delta = 1}) or to 2 (\code{fix_delta = 2}); the latter
 #'two are specific submodels of a FIAPARCH.
+#'@param skip_vcov a logical indicating whether or not to skip the computation
+#'of the variance-covariance matrix of the parameter estimators and therefore
+#'also standard error computation.
 #'
 #'@details
 #'Consider a FIAPARCH(\eqn{p, d, q}) with constant asymmetry term \eqn{\gamma} regardless
@@ -419,7 +422,7 @@ fiaparch <- function(rt, orders = c(1, 1),
                         control_nonpar = list(), mean_after_nonpar = FALSE,
                         parallel = TRUE, ncores = max(1, future::availableCores() - 1),
                         trunc = "none", presample = 50, Prange = c(1, 5),
-                        fix_delta = c(NA, 1, 2)) {
+                        fix_delta = c(NA, 1, 2), skip_vcov = FALSE) {
 
   inp_args <- mget(names(formals()), envir = environment())
 
