@@ -740,6 +740,21 @@ methods::setMethod("predict_internal", "fEGarch_fit",
 #'@param trunc the truncation setting for the infinite-order
 #'polynomial of long-memory model parts; the default uses
 #'the setting from the fitted input object \code{object}.
+#'@param refit_after a positive integer indicatinmg the number
+#'of observations of the test
+#'set, after which the model should be refitted regularly; for the
+#'default \code{NULL} no refitting is done.
+#'@param steady_window a logical; if \code{FALSE}, the refitting
+#'will use all observations back to the first observation time point;
+#'for \code{TRUE}, the number of training observations is held constant,
+#'so that the window of training observations is shifted at every
+#'refitting.
+#'@param parallel a logical, indicating whether the models and forecasts
+#'should be produced in parallel for a speed boost.
+#'@param ncores the number of cores to use for parallel computations.
+#'@param fitting_args a list with further fitting arguments for any of the
+#'main fitting functions from this package; useful in the refitting, when
+#'other settings than the default need to be implemented.
 #'@param ... currently without use and included for compatibility
 #'with generics.
 #'
@@ -762,7 +777,18 @@ methods::setMethod("predict_internal", "fEGarch_fit",
 #'forecasts (starting at the tenth test time point).
 #'
 #'Refitting of models during the rolling point forecast procedure
-#'is currently not yet available.
+#'is available with package version 1.0.4 for GARCH-type models.
+#'\code{refit_after} defines
+#'after how many test observations the model should be refitted every time.
+#'\code{steady_window} addresses whether in refitting models the training
+#'observations should always start at the very first observation time
+#'point available, i.e. \code{steady_window = FALSE}, or whether the initial
+#'number of training observations should be kept throughout when refitting, so
+#'that the window width of training observations is kept but shifted when
+#'refitting, i.e. \code{steady_window = TRUE}. The behavior of
+#'\code{predict_roll} from package version 1.0.3 or earlier, i.e. obtaining
+#'rolling forecasts without model refitting, can be reproduced
+#'by setting \code{refit_after = NULL}.
 #'
 #'@return
 #'Returns an object of class \code{"fEGarch_forecast"} that has the
@@ -771,8 +797,10 @@ methods::setMethod("predict_internal", "fEGarch_fit",
 #'means, respectively. If the training series saved in \code{object}
 #'has a special time series formatting like \code{"zoo"} or \code{"ts"},
 #'the formatting is adopted accordingly to these numeric
-#'output series. A third slot \code{@model} is the fitted model
-#'input object \code{object}.
+#'output series. A third slot \code{@model} is either the fitted model
+#'input object \code{object} (when refitting is disabled) or a list as a
+#'collection of fitted model objects of class \code{"fEGarch_fit"}
+#'(when refitting is enabled).
 #'
 #'@export
 #'
@@ -788,8 +816,11 @@ methods::setMethod("predict_internal", "fEGarch_fit",
 #'fcast1
 #'
 #'# Rolling one-step forecasts (EGARCH with cond. normal distr.)
+#'# (Parallel computation disabled for better compatibility with
+#'# CRAN checks)
 #'model2 <- fEGarch(spec = egarch_spec(), rt, n_test = 250)
-#'fcast2 <- predict_roll(model2, step_size = 1)
+#'fcast2 <- predict_roll(model2, step_size = 1, refit_after = 50,
+#'                       parallel = FALSE)
 #'fcast2
 #'
 methods::setMethod("predict", "fEGarch_fit",
@@ -824,11 +855,7 @@ methods::setMethod("predict", "fEGarch_fit",
   }
 )
 
-#'@export
-#'
-#'@rdname forecasting-methods
-#'
-methods::setMethod("predict_roll", "fEGarch_fit",
+methods::setMethod("predict_roll_internal", "fEGarch_fit",
   function(object, step_size = 1, trunc = NULL, ...) {
 
     # Get truncation for long-memory models from fitted object
@@ -943,6 +970,237 @@ methods::setMethod("predict_roll", "fEGarch_fit",
       model = object
     )
 
+})
+
+#'@export
+#'
+#'@rdname forecasting-methods
+#'
+methods::setMethod("predict_roll", "fEGarch_fit",
+  function(object, step_size = 1, trunc = NULL, refit_after = NULL, steady_window = FALSE, parallel = TRUE, ncores = max(1, future::availableCores() - 1), fitting_args = list(), ...) {
+
+    stopifnot("step_size must be an integer that is at least 1." =
+                !is.na(step_size) && is.numeric(step_size) && step_size >= 1)
+    stopifnot("trunc must be a positive integer or NULL" =
+                is.null(trunc) || (!is.na(trunc) && is.numeric(trunc) && trunc >= 1))
+    stopifnot("refit_after must be a positive integer or NULL" =
+                is.null(refit_after) || (!is.na(refit_after) && is.numeric(refit_after) && refit_after >= 1))
+    stopifnot("steady_window must be a logical value" =
+                !is.na(steady_window) && is.logical(steady_window))
+    stopifnot("parallel must be a logical value" =
+                !is.na(parallel) && is.logical(parallel))
+    stopifnot("ncores must be a positive integer" =
+                !is.na(ncores) && is.numeric(ncores) && ncores >= 1)
+
+
+    fitting_args[["parallel"]] <- FALSE
+    fitting_args[["skip_vcov"]] <- TRUE
+
+    train <- object@rt
+    test <- object@test_obs
+
+    n_train <- length(train)
+    n_test <- length(test)
+
+    # Get nonparametric settings
+    if (!is.null(object@nonpar_model)) {
+      bwidth <- object@nonpar_model$b0
+      polyord <- object@nonpar_model$p
+      kernord <- object@nonpar_model$mu
+      bound_meth <- object@nonpar_model$bb
+      bound_meth <- ifelse(bound_meth == 1, "extend", "shorten")
+      np_spec <- locpol_spec(poly_order = polyord, kernel_order = kernord,
+                             boundary_method = bound_meth, bwidth = bwidth)
+      use_nonpar <- TRUE
+      bwidth_abs <- n_train * bwidth
+    } else { # irrelevant settings
+      np_spec <- locpol_spec(poly_order = 1, kernel_order = 1,
+                             boundary_method = "extend", bwidth = 0.2)
+      use_nonpar <- FALSE
+      bwidth_abs <- 5   # irrelevant
+    }
+
+    fitting_args[["use_nonpar"]] <- use_nonpar
+
+    # If no refitting is needed, run internal predict roll function once
+    # and return result
+    if (is.null(refit_after) || refit_after >= n_test) {
+
+      return(predict_roll_internal(object, step_size = step_size, trunc = trunc))
+
+    }
+
+    # Ends of sub-training + test obs.
+    splits_test <- seq(from = n_train + step_size - 1, to = n_train + n_test, by = refit_after)
+
+
+    obs_all <- c(unclass(train), unclass(test))
+
+    n_all <- length(obs_all)
+
+    if (utils::tail(splits_test, 1) >= n_all) {
+
+      splits_test <- splits_test[-length(splits_test)]
+
+    }
+
+    obs_end <- c(splits_test, n_all)
+    obs_sets <- vector(mode = "list", length = length(splits_test))
+    for (i in 1:length(splits_test)) {
+      obs_sets[[i]] <- obs_all[(1 + (i - 1) * refit_after * steady_window):obs_end[[i + 1]]]
+    }
+
+    cdist <- object@cond_dist
+    m_order <- object@orders
+    mspec <- object@meanspec
+    lmemo <- object@long_memo
+    fitting_args[["meanspec"]] <- mspec
+
+    if (inherits(object, "fEGarch_fit_egarch")) {
+      pwers <- object@powers
+      modu <- object@modulus
+    } else if (inherits(object, "fEGarch_fit_loggarch")) { # irrelevant settings
+      pwers <- c(0, 0)
+      modu <- c(FALSE, FALSE)
+    }
+    trunc_setting <- object@trunc
+
+    test_sizes <- diff(obs_end)
+
+    # Get selector for corresponding fitting subclass
+    selector <- vapply(
+      c("fEGarch_fit_egarch", "fEGarch_fit_loggarch", "fEGarch_fit_aparch",
+        "fEGarch_fit_fiaparch", "fEGarch_fit_garch", "fEGarch_fit_figarch",
+        "fEGarch_fit_tgarch", "fEGarch_fit_fitgarch", "fEGarch_fit_gjrgarch",
+        "fEGarch_fit_figjrgarch"),
+      FUN = function(.class, object) {
+        inherits(object, what = .class)
+      },
+      FUN.VALUE = logical(1),
+      object = object
+    )
+
+    adj_fitting_args <- function(rt, n_test, fitting_args) {
+      fitting_args[["rt"]] <- rt
+      fitting_args[["n_test"]] <- n_test + step_size - 1
+      np_spec@bwidth <- bwidth_abs / (length(rt) - fitting_args[["n_test"]])
+      fitting_args[["nonparspec"]] <- np_spec
+      fitting_args
+    }
+
+    fc_fun_fegarch <- function(rt, n_test, mtype) {
+      fitting_args <- adj_fitting_args(rt = rt, n_test = n_test, fitting_args = fitting_args)
+      fitting_args[["spec"]] <- fEGarch_spec(model_type = mtype, orders = m_order, long_memo = lmemo, cond_dist = cdist, powers = pwers, modulus = modu)
+      do.call(what = fEGarch, args = fitting_args) %>% predict_roll_internal(step_size = step_size, trunc = trunc)
+    }
+
+    fc_fun_other_garch <- function(rt, n_test, fit_fun) {
+      fitting_args <- adj_fitting_args(rt = rt, n_test = n_test, fitting_args = fitting_args)
+      fitting_args[["cond_dist"]] <- cdist
+      do.call(fit_fun, fitting_args) %>% predict_roll_internal()
+    }
+
+    # Select the proper fitting and forecasting function
+    fc_fun <- list(
+      "fEGarch_fit_egarch" = function(rt, n_test) {
+        fc_fun_fegarch(rt = rt, n_test = n_test, mtype = "egarch")
+      },
+      "fEGarch_fit_loggarch" = function(rt, n_test) {
+        fc_fun_fegarch(rt = rt, n_test = n_test, mtype = "loggarch")
+      },
+      "fEGarch_fit_aparch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = aparch)
+      },
+      "fEGarch_fit_fiaparch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = fiaparch)
+      },
+      "fEGarch_fit_garch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = garch)
+      },
+      "fEGarch_fit_figarch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = figarch)
+      },
+      "fEGarch_fit_tgarch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = tgarch)
+      },
+      "fEGarch_fit_fitgarch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = fitgarch)
+      },
+      "fEGarch_fit_gjrgarch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = gjrgarch)
+      },
+      "fEGarch_fit_figjrgarch" = function(rt, n_test) {
+        fc_fun_other_garch(rt = rt, n_test = n_test, fit_fun = figjrgarch)
+      }
+    )[selector][[1]]
+
+    # Set up parallel computations
+    oldplan <- future::plan()
+    if (parallel) {
+      future::plan(future::multisession, workers = ncores)
+
+      on.exit(expr = {future::plan(oldplan)}, add = TRUE, after = TRUE)
+    }
+
+
+    # Fit models in parallel and obtain the rolling forecasts
+    out <- furrr::future_map2(obs_sets, test_sizes, function(.x, .y, fc_fun) {
+      fc_fun(rt = .x, n_test = .y)
+    }, fc_fun = fc_fun, .options = furrr::furrr_options(seed = NULL), .progress = FALSE)
+
+
+    future::plan(oldplan)
+
+    # Get forecasted values
+    sigmas <- lapply(
+      out,
+      function(.x) {
+        stats::na.omit(unclass(.x@sigt))
+      }
+    ) %>%
+      unlist()
+    if (step_size >= 2) {
+      sigmas <- c(rep(NA, step_size - 1), sigmas)
+    }
+
+    means <- lapply(
+      out,
+      function(.x) {
+        stats::na.omit(unclass(.x@cmeans))
+      }
+    ) %>%
+      unlist()
+    if (step_size >= 2) {
+      means <- c(rep(NA, step_size - 1), means)
+    }
+
+    # Get individual models for the various sections
+    models <- lapply(
+      out,
+      function(.x) {
+        .x@model
+      }
+    )
+
+    # Format output series
+    series_out <- format_applier_ts(
+      rt = test,
+      list_of_ts = list(
+        "cmeans" = means,
+        "sigt" = sigmas
+      )
+    )
+
+    cmeans_out <- series_out$cmeans
+    sigt_out <- series_out$sigt
+
+    # Return object
+    fEGarch_forecast(
+      cmeans = cmeans_out,
+      sigt = sigt_out,
+      model = models
+    )
+
   }
 )
 
