@@ -107,7 +107,9 @@ setGeneric("measure_risk", function(object, measure = c("VaR", "ES"), level = c(
 #'window.zoo <- get("window.zoo", envir = asNamespace("zoo"))
 #'rt <- window.zoo(SP500, end = "2002-12-31")
 #'model2 <- fEGarch(egarch_spec(), rt, n_test = 250)
-#'fcast <- predict_roll(model2)
+#'# parallel = FALSE only for better compatibility with CRAN checks;
+#'# should be set to TRUE for speed boost
+#'fcast <- predict_roll(model2, parallel = FALSE)
 #'risk2 <- measure_risk(fcast, measure = c("VaR", "ES"), level = c(0.95, 0.975, 0.99))
 #'risk2
 #'
@@ -119,7 +121,9 @@ setGeneric("measure_risk", function(object, measure = c("VaR", "ES"), level = c(
 #'window.zoo <- get("window.zoo", envir = asNamespace("zoo"))
 #'rt <- window.zoo(SP500, end = "2005-12-31")
 #'model <- fEGarch(egarch_spec(), rt, n_test = 250)
-#'fc <- model %>% predict_roll()
+#'# parallel = FALSE only for better compatibility with CRAN checks;
+#'# should be set to TRUE for speed boost
+#'fc <- model %>% predict_roll(refit_after = 50, parallel = FALSE)
 #'
 #'test_obs <- model@test_obs   # Test observations
 #'sigt <- fc@sigt              # Conditional volatility forecasts
@@ -212,15 +216,125 @@ setMethod("measure_risk", "fEGarch_fit",
 setMethod("measure_risk", "fEGarch_forecast",
   function(object, measure = c("VaR", "ES"), level = c(0.975, 0.99), ...) {
 
-    object2 <- object@model
-    object2@sigt <- object@sigt
-    object2@cmeans <- object@cmeans
+    # Number of fitted models
+    n_fits <- length(object@model)
 
-    func <- methods::selectMethod("measure_risk", "fEGarch_fit")
-    out <- func(object = object2, measure = measure, level = level, ...)
-    out@observations <- object2@test_obs
-    out@sigt <- object2@sigt
-    out@cmeans <- object2@cmeans
+    if (n_fits == 1) {
+
+      object2 <- object@model
+      object2@sigt <- object@sigt
+      object2@cmeans <- object@cmeans
+
+      func <- methods::selectMethod("measure_risk", "fEGarch_fit")
+      out <- func(object = object2, measure = measure, level = level, ...)
+      out@observations <- object2@test_obs
+      out@model <- object@model
+
+
+    } else if (n_fits >= 2) {
+
+      # Get length of test series for each submodel
+      len_sub <- vapply(
+        object@model,
+        function(.x) {
+          length(.x@test_obs)
+        },
+        FUN.VALUE = numeric(1)
+      )
+
+
+      out_s <- lapply(
+        1:n_fits,
+        function(.x, len_sub, level, measure) {
+          model <- object@model[[.x]]
+          idx_l <- (.x - 1) * len_sub[[1]] + 1
+          idx_u <- (.x - 1) * len_sub[[1]] + len_sub[[.x]]
+          sigmas <- unclass(object@sigt)[idx_l:idx_u]
+          means <- unclass(object@cmeans)[idx_l:idx_u]
+
+          model@sigt <- sigmas
+          model@cmeans <- means
+          func <- methods::selectMethod("measure_risk", "fEGarch_fit")
+          out <- func(object = model, measure = measure, level = level, ...)
+          out@observations <- model@test_obs
+          out
+        },
+        len_sub = len_sub, level = level, measure = measure
+      )
+
+    # As formatting reference
+    sigmas <- object@sigt
+
+    test_obs <- lapply(
+      out_s, function(.x) {unclass(.x@observations)}
+    ) %>%
+      unlist()
+
+    form <- format_applier_ts(
+      rt = sigmas,
+      list_of_ts = list(
+        "test_obs" = test_obs
+      )
+    )
+
+    test_obs <- form$test_obs
+
+    VaR <- vector(mode = "list", length = length(level))
+    names(VaR) <- paste0("VaR", level)
+
+    ES <- vector(mode = "list", length = length(level))
+    names(ES) <- paste0("ES", level)
+
+    for (i in 1:length(level)) {
+
+      lvl <- level[[i]]
+      nam_var <- paste0("VaR", lvl)
+      nam_es <- paste0("ES", lvl)
+
+      VaR_i <- lapply(out_s, function(.x, nam_var) {
+        unclass(.x@measures$VaR[[nam_var]])
+      }, nam_var = nam_var) %>%
+        unlist()
+
+      ES_i <- lapply(out_s, function(.x, nam_es) {
+        unclass(.x@measures$ES[[nam_es]])
+      }, nam_es = nam_es) %>%
+        unlist()
+
+      form <- format_applier_ts(
+        rt = sigmas,
+        list_of_ts = list(
+          "VaR" = VaR_i,
+          "ES" = ES_i
+        )
+      )
+
+      VaR[[nam_var]] <- form$VaR
+      ES[[nam_es]] <- form$ES
+
+    }
+
+    if (!("VaR" %in% measure)) {
+      VaR <- list()
+    }
+    if (!("ES" %in% measure)) {
+      ES <- list()
+    }
+
+      measures <- list(
+        VaR = VaR,
+        ES = ES
+      )
+
+      out <- out_s[[1]]
+      out@measures <- measures
+      out@observations <- test_obs
+      out@model <- object@model
+
+    }
+
+    out@sigt <- object@sigt
+    out@cmeans <- object@cmeans
     out
   }
 )
@@ -369,7 +483,9 @@ zone_selector <- function(p) {
 #'window.zoo <- get("window.zoo", envir = asNamespace("zoo"))
 #'rt <- window.zoo(SP500, end = "2002-12-31")
 #'model <- fEGarch(egarch_spec(), rt, n_test = 250)
-#'fcast <- predict_roll(model)
+#'# parallel = FALSE for better compatibility with CRAN checks;
+#'# should be set to TRUE for speed boost
+#'fcast <- predict_roll(model, refit_after = 50, parallel = FALSE)
 #'risk <- measure_risk(fcast, measure = c("VaR", "ES"), level = c(0.95, 0.975, 0.99))
 #'trafflight_test(risk)
 #'cov_tests(risk)
@@ -381,15 +497,18 @@ setMethod("trafflight_test", "fEGarch_risk",
   obs <- object@observations
   n <- length(obs)
 
-  slotnames <- methods::slotNames(object@model)
+  if (!is.list(object@model)) {object@model <- list(object@model)}
+  slotnames <- methods::slotNames(object@model[[1]])
+  n_te_base <- length(object@model[[1]]@test_obs)
   cond_d <- if ("cond_dist" %in% slotnames) {
-    object@model@cond_dist
+    object@model[[1]]@cond_dist
   } else if ("dist" %in% slotnames) {
-    object@model@dist
+    object@model[[1]]@dist
   } else {
     stop("Neither slot @cond_dist nor @dist to be found in model.")
   }
 
+
   par_name <- switch(
     cond_d,
     "norm" = "df",    # irrelevant
@@ -401,17 +520,6 @@ setMethod("trafflight_test", "fEGarch_risk",
     "ald" = "P",
     "sald" = "P"
   )
-  shape <- if (cond_d %in% c("norm", "snorm")) {
-    0
-  } else {
-    object@model@pars[[par_name]]
-  }
-
-  skew <- if (cond_d %in% c("snorm", "sstd", "sged", "sald")) {
-    object@model@pars[["skew"]]
-  } else {
-    0
-  }
 
   VaR_names <- names(object@measures$VaR)
   ES_names <- names(object@measures$ES)
@@ -443,42 +551,65 @@ setMethod("trafflight_test", "fEGarch_risk",
 
   if (n_ES > 0 && n_VaR > 0) {
 
-    cmeans <- object@cmeans
-    sigt <- object@sigt
+    cmeans <- unclass(object@cmeans)
+    sigt <- unclass(object@sigt)
+
+    et <- (unclass(obs) - cmeans) / sigt     # standardized residuals
 
-    et <- (obs - cmeans) / sigt     # standardized residuals
     args <- list(
-      dfun = fun1_selector(cond_d),
-      skew = skew,
-      shape = shape
+      dfun = fun1_selector(cond_d)
     )
 
-
     names(list_ES) <- ES_names
     for (i in 1:n_ES) {
       conf_lvl <- as.numeric(substring(ES_names[[i]], 3))
+      alpha <- conf_lvl
       check_VaR <- paste0("VaR", conf_lvl)
       if (!(check_VaR %in% VaR_names)) {
         message(paste0(ES_names, "cannot be analyzed because ", check_VaR, "is missing from input object"))
         next
       }
 
-      pot_idx <- obs < object@measures$VaR[[check_VaR]]
-      alpha <- conf_lvl
-      sum_severity <- if (sum(pot_idx) > 0) {
-        et_sub <- zoo::coredata(et[pot_idx])
-        args[["x"]] <- et_sub
-        nprobs <- do.call(what = cdf_fun, args = args)
-        probs <- 1 - nprobs
+      sum_severity <- 0   # default
+
+      for (j in 1:length(object@model)) {
+
+          shape <- if (cond_d %in% c("norm", "snorm")) {
+            0
+          } else {
+            object@model[[j]]@pars[[par_name]]
+          }
+
+          skew <- if (cond_d %in% c("snorm", "sstd", "sged", "sald")) {
+            object@model[[j]]@pars[["skew"]]
+          } else {
+            0
+          }
+
+          args[["skew"]] <- skew
+          args[["shape"]] <- shape
+
+          n_te <- length(object@model[[j]]@test_obs)
+          idx_lo <- (j - 1) * n_te_base + 1
+          idx_up <- (j - 1) * n_te_base + n_te
+
+          pot_idx <- unclass(obs)[idx_lo:idx_up] < unclass(object@measures$VaR[[check_VaR]])[idx_lo:idx_up]
+
+          if (sum(pot_idx) > 0) {
+            et_sub <- zoo::coredata(et[idx_lo:idx_up][pot_idx])
+            args[["x"]] <- et_sub
+            nprobs <- do.call(what = cdf_fun, args = args)
+            probs <- 1 - nprobs
+
+            sum_severity <- sum_severity + sum((probs - alpha) / (1 - alpha))
+          }
 
-        sum((probs - alpha) / (1 - alpha))
-      } else {
-        0
       }
-        mean_ES <- 0.5 * (1 - alpha) * n
-        var_ES <- n * (1 - alpha) * ((1 + 3 * alpha) / 12)
 
-        prob <- pnorm(q = sum_severity, mean = mean_ES, sd = sqrt(var_ES))
+      mean_ES <- 0.5 * (1 - alpha) * n
+      var_ES <- n * (1 - alpha) * ((1 + 3 * alpha) / 12)
+
+      prob <- pnorm(q = sum_severity, mean = mean_ES, sd = sqrt(var_ES))
 
       list_ES[[i]] <- list(
         conf_lvl = conf_lvl,
